// TPromptEdit
// V1.01 12-OCT-1998
// (c) Stas Malinovski, 1998.
// E-mail: stasm@mail.ru
// Freeware

// Also, if you add features to TPromptEdit, please, let me know. I'll be
// grateful to anybody will help me make TPromptEdit bug-free, powerful
// and still available for free.

// I you are not a programming-guru and you need some extra features send
// me you request. Please, ask only for general-purpose features that may
// be useful to others.

// I'd like any comments on this unit be posted to stasm@mail.ru

unit PrmtEdit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
   Dialogs, StdCtrls, Mask;

type
  TPromptStyle = (psNone, psAlways, psOnlyIfEmpty);

  TCustomPromptEdit = class(TCustomMaskEdit)
  public
    constructor Create(AOwner: TComponent); override;
  private
    FPrompt: string;
    FPromptStyle: TPromptStyle;
    FParentPromptColor: Boolean;
    FPromptColor: TColor;
    FPromptTextColor: TColor;
    FPromptBorder: TBorderStyle;
    FBorderSize: TSize;
    FBorderFlags: integer;
    FDC: HDC;
    procedure SetPrompt(const Value: string);
    procedure SetPromptStyle(Value: TPromptStyle);
    procedure SetParentPromptColor(Value: Boolean);
    procedure SetPromptColor(Value: TColor);
    procedure SetPromptTextColor(Value: TColor);
    procedure SetPromptBorder(Value: TBorderStyle);
  private
    procedure WMSetFocus(var Message: TMessage); message WM_SETFOCUS;
    procedure WMKillFocus(var Message: TMessage); message WM_KILLFOCUS;
  protected
    procedure WndProc(var Message: TMessage); override;
    procedure Paint; virtual;
    procedure CreateWnd; override;
    procedure ShowBorder(Value: Boolean); virtual;
    function CalcTextPosition: TPoint;
    procedure CalcBorderSize; virtual;
    procedure RedrawIfNeeded;
    property DC: HDC read FDC;
  protected
    property Prompt: string
      read FPrompt write SetPrompt;
    property PromptStyle: TPromptStyle
      read FPromptStyle write SetPromptStyle default psOnlyIfEmpty;
    property ParentPromptColor: Boolean
      read FParentPromptColor write SetParentPromptColor default False;
    property PromptColor: TColor
      read FPromptColor write SetPromptColor default clWhite;
    property PromptTextColor: TColor
      read FPromptTextColor write SetPromptTextColor default clGray;
    property PromptBorder: TBorderStyle
      read FPromptBorder write SetPromptBorder default bsNone;
  end;

  TPromptEdit = class(TCustomPromptEdit)
  published
    property Prompt;
    property PromptStyle;
    property ParentPromptColor;
    property PromptColor;
    property PromptTextColor;
    property PromptBorder;
    property AutoSelect;
    property AutoSize;
    property BorderStyle;
    property CharCase;
    property Color;
    property Ctl3D;
    property DragCursor;
    property DragMode;
    property EditMask;
    property Enabled;
    property Font;
    property HideSelection;
    property ImeMode;
    property ImeName;
    property MaxLength;
    property OEMConvert;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PasswordChar;
    property PopupMenu;
    property ReadOnly;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Text;
    property Visible;
    property OnChange;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDrag;
  end;

procedure Register;

implementation

const
  DefaultPrompt = '<prompt text>';

procedure Register;
begin
  RegisterComponents('Samples',[TPromptEdit]);
end;

function WidthOf(const R: TRect): integer; assembler;
asm
  push  edi
  mov   edi,R
  mov   eax,[edi].TRect.Right
  sub   eax,[edi].TRect.Left
  pop   edi
end;

function HeightOf(const R: TRect): integer; assembler;
asm
  push  edi
  mov   edi,R
  mov   eax,[edi].TRect.Bottom
  sub   eax,[edi].TRect.Top
  pop   edi
end;

constructor TCustomPromptEdit.Create;
begin
  inherited Create(AOwner);
  FPrompt:= DefaultPrompt;
  FPromptStyle:= psOnlyIfEmpty;
  FParentPromptColor:= False;
  FPromptColor:= clWhite;
  FPromptTextColor:= clGray;
  FPromptBorder:= bsNone;
end;

procedure TCustomPromptEdit.SetPrompt;
begin
  FPrompt:= Value;
  RedrawIfNeeded;
end;

procedure TCustomPromptEdit.SetPromptStyle;
var
  s: TPromptStyle;
begin
  if Value <> FPromptStyle then
    begin
      s:= FPromptStyle;
      FPromptStyle:= Value;
      case FPromptStyle of
       psNone: ShowBorder(False);
       psAlways,psOnlyIfEmpty:
        if s = psNone then
          begin
            CalcBorderSize;
            ShowBorder(True);
          end
        else Invalidate;
      end;
   end;
end;

procedure TCustomPromptEdit.SetParentPromptColor;
begin
  if Value <> FParentPromptColor then
    begin
      FParentPromptColor:= Value;
      RedrawIfNeeded;
    end;
end;

procedure TCustomPromptEdit.SetPromptColor;
begin
  if Value <> FPromptColor then
    begin
      FPromptColor:= Value;
      RedrawIfNeeded;
    end;
end;

procedure TCustomPromptEdit.SetPromptTextColor;
begin
  if Value <> FPromptTextColor then
    begin
      FPromptTextColor:= Value;
      RedrawIfNeeded;
    end;
end;

procedure TCustomPromptEdit.SetPromptBorder;
begin
  if Value <> FPromptBorder then
    begin
      FPromptBorder:= Value;
      RedrawIfNeeded;
    end;
end;

procedure TCustomPromptEdit.WMSetFocus;
begin
  inherited;
  if (PromptStyle <> psNone) then
    ShowBorder(False);
end;

procedure TCustomPromptEdit.WMKillFocus;
begin
  inherited;
  if (PromptStyle <> psNone) then
    begin
      CalcBorderSize;
      ShowBorder(True);
    end;
end;

function TCustomPromptEdit.CalcTextPosition;
var
  SaveFont: HFont;
  I: Integer;
  SysMetrics, Metrics: TTextMetric;
begin
  if NewStyleControls then
  begin
    if BorderStyle = bsNone then I:= 0 else
      if Ctl3D then I:= 1 else I:= 2;
    Result.X:= SendMessage(Handle,EM_GETMARGINS,0,0) and $0000FFFF + I;
    Result.Y:= I;
  end else
  begin
    if BorderStyle = bsNone then I:= 0 else
    begin
      GetTextMetrics(DC, SysMetrics);
      SaveFont := SelectObject(DC, Font.Handle);
      GetTextMetrics(DC, Metrics);
      SelectObject(DC, SaveFont);
      I := SysMetrics.tmHeight;
      if I > Metrics.tmHeight then I := Metrics.tmHeight;
      I := I div 4;
    end;
    Result.X:= I;
    Result.Y:= I;
  end;
end;

procedure TCustomPromptEdit.CalcBorderSize;
begin
  if GetWindowLong(Handle,GWL_EXSTYLE) and WS_EX_CLIENTEDGE <> 0 then
    begin
      FBorderFlags:= EDGE_SUNKEN;
      FBorderSize.cx:=$2;
      FBorderSize.cy:=$2;
    end
  else
    begin
      FBorderFlags:=0;
      FillChar(FBorderSize,SizeOf(TSize),0);
    end;
end;

procedure TCustomPromptEdit.RedrawIfNeeded;
begin
  if (PromptStyle <> psNone) and (not Focused) and
   IsWindowVisible(Handle) and (not (csDesigning in ComponentState)) then
    Invalidate;
end;

procedure TCustomPromptEdit.WndProc;
var
  PS: TPaintStruct;
begin
  if (Message.Msg = WM_PAINT) and (PromptStyle <> psNone) and (not Focused)
   and (not (csDesigning in ComponentState)) then
    begin
      BeginPaint(Handle,PS);
      try
        FDC:= PS.hdc;
        Paint;
      finally
        EndPaint(Handle,PS);
      end;
    end
   else Inherited WndProc(Message);
end;

procedure TCustomPromptEdit.Paint;
var
  Brush,SaveBrush: HBrush;
  R: TRect;
  F: HFont;
  s: string;
  P: TPoint;
begin
  R:= ClientRect;
  if ParentPromptColor then
    Brush:= SendMessage(Parent.Handle,CN_CTLCOLOREDIT,DC,0)
  else
    begin
      Brush:= CreateSolidBrush(PromptColor);
      SetTextColor(DC,PromptTextColor);
      SetBkMode(DC,Windows.TRANSPARENT);
    end;
  if PromptBorder = bsNone then
    FillRect(DC,R,Brush)
  else
    begin
      SaveBrush:= SelectObject(DC,Brush);
      Rectangle(DC,0,0,WidthOf(R),HeightOf(R));
      SelectObject(DC,SaveBrush);
    end;
  if not ParentPromptColor then
    DeleteObject(Brush);
  if PromptStyle = psOnlyIfEmpty then
    if length(Text) = 0 then
      s:= Prompt
    else s:= Text
  else s:= Prompt;
  F:= SelectObject(DC,Font.Handle);
  P:= CalcTextPosition;
  TextOut(DC,P.X+FBorderSize.cx,P.Y+FBorderSize.cy,PChar(s),length(s));
  SelectObject(DC,F);
end;

procedure TCustomPromptEdit.CreateWnd;
begin
  inherited CreateWnd;
  if (PromptStyle <> psNone) and (not (csDesigning in ComponentState)) then
    begin
      CalcBorderSize;
      ShowBorder(True);
    end;
end;

procedure TCustomPromptEdit.ShowBorder;
var
  R: TRect;
  Style: integer;
begin
  if Value then
    begin
      R:= ClientRect;
      Inc(R.Right,FBorderSize.cx*2);
      Inc(R.Bottom,FBorderSize.cy*2);
      if FBorderFlags and EDGE_SUNKEN <> 0 then
        begin
          Style:= GetWindowLong(Handle,GWL_EXSTYLE);
          Style:= Style and not WS_EX_CLIENTEDGE;
          SetWindowLong(Handle,GWL_EXSTYLE,Style);
        end;
      SetWindowPos(Handle,HWND_TOP,0,0,WidthOf(R),HeightOf(R),
        SWP_NOMOVE or SWP_NOZORDER or SWP_NOACTIVATE or SWP_DRAWFRAME);
    end
  else
    begin
      if FBorderFlags and EDGE_SUNKEN <> 0 then
        begin
          Style:= GetWindowLong(Handle,GWL_EXSTYLE);
          Style:= Style or WS_EX_CLIENTEDGE;
          SetWindowLong(Handle,GWL_EXSTYLE,Style);
        end;
      SetWindowPos(Handle,HWND_TOP,0,0,0,0, SWP_NOSIZE or
        SWP_NOMOVE or SWP_NOZORDER or SWP_DRAWFRAME);
    end;
  Invalidate;
end;

end.
